/*
﻿###################################################################################################################################################################
#                               Software for Arduino Pro Mini,Uno/Nano, s CPU 328P,168P,5V,16MHz   autor:Ondrej Nadrchal email: onadr@seznam.cz                   #  
#     Embeded SW pro modul S-COM rozsireny prijimac a navestni dekoder                                                                                            #
#     Ver:1.0.                                                                                                                                                    #
#     august 2019                                                                                                                                                 #
#     Licenced: GNU                                                                                                                                               #
###################################################################################################################################################################

Prijimac signalu s-com a navestni dekoder je 100% kompatibilnimi se standardnimi S-COM prijimaci.
Tento dekoder obsahuje vsechny svetelne navesti ze vsech epoch CSD/CD do rychlosti 100km/h.
Nastaveni maximalniho jasu jednotlivych LED diod od 6%-100% a ulozeni v EEPROM (pres s-com signal s pomoci jineho arduina).
Plynuly nabeh a dosvit svetel jako u zarovek.
Moznost pridani vlastnich navesti a nebo modifikace pro jine navestni soustavy (vyzaduje upravu casti zdrojoveho kodu.
Pri kazdem spravne prijatem paketu kratce blikne LED na vystupu 13.
Tento prijimac/dekoder vyzaduje mezeru mezi S/COM pakety pouze desetinasobek delky bitu (Tb), nemusi se tedy vkladat min 200ms.

zapojeni LED diod navestidla:
-anody diod (kladny pol) spojene a zapojene na +5V arduina (nebo na +3,3V) 
-katody diod (zaporny pol) zapojene na jednotlive vystupy arduina pres rezistor (obvykle >1kohm)
  -horni zluta 2
  -zelena 3
  -cervena 4
  -bila 5
  -dolni zluta 6
  -modra 7
  -horni pruh/prosvetleny symbol 8
  -dolni pruh 9
ostatni zapojeni
  -napajeni 12V na napajeci vstupy arduina (podle schematu pouziteho modulu, VIN, RAW, ....)
  -napajeni zem (GND) na vyvod GND arduina
  -vstup s-com signalu 12
  -pro mod predvesti spojit pin 11 se zemi (GND)
*/
//------------ Obecne definice funkcionality, ktere lze bez problemu zmenit
#define DELAY 150  // doba nabehu nebo dosvitu LED pri zmene v ms   (minimalni hodnota 1, maximalni hodnota 250)
#define SETUPENABLE 1  // hodnota 1 povoli funkci setupu a ulozeni jasu jednotlivych LED do EEPROM a obnovu z ni pri kazdem zapnuti, jina hodnota setup zakaze a hodnoty jasu se budou urcovat pouze z pole MaxIntensity[]
#define DEFAULTSIGNAL 0    //signal s timto SCom kodem se bude zobrazovat po zapnuti, nez se prijme prvni platny kod
#define MODPREDVEST 11    //cislo pinu pro mod predvesti, jeho pripojenim na zem (LOW) se aktivuje zobrazovani signalu predvesti, slouzi pro zajisteni kompatibility se stavajicimi S-COM dekodery
#define EEPROMBASEADR 32  //adresa v EEPROM, kde jsou ulozene hodnoty pro setup, pri zmene je potreba znovu provest setup a ulozit ho,mozne pouzit hodnotu 0 az max. adresa Eeprom-10
#define FORCEEEPROMWRITE 0 //pokud je hodnota 1, tak se defaultni nastaveni intensit vzdy po zapnuti zapise do EEPROM (prepise posledni nastaveni), prakticky pouzitelne pro ucely ladeni
byte MaxIntensity[8]={100,100,100,100,100,100,100,100}; //maximalni hodnota jasu v % u jednotlivych LED,hodnota muze byt min=6 max=100 poradi LED odpovida tomu v OUTPIN[]
//------------ END Obecne definice funkcionality, ktere lze bez problemu zmenit
// ----------- LED display definition
#define MAXBRIGHT 100 // Maximalni jas LED v %, min. hodnota 2 a max hodnota 100
#define PWMSTEPS 50  //pocet kroku jedne sekvence PWM (nemenit, pokud nevite co delate)
const byte MaxLEDInt=(byte)MAXBRIGHT/2;
// ----------- END LED display definition
//------------LED pin definition
#define UPPERYELLOW 2   //cislo pinu pro horni zluta
#define GREEN 3         //cislo pinu pro zelena
#define RED 4           //cislo pinu pro cervena
#define WHITE 5         //cislo pinu pro bila
#define LOWERYELLOW 6   //cislo pinu pro dolni zluta
#define BLUE 7          //cislo pinu pro modra
#define UPPERVARIABLE 8 //cislo pinu pro horni pruh nebo jiny prosvetlovany symbol
#define LOWERSTRIP 9    //cislo pinu pro dolni pruh
const byte OUTPIN[8]={UPPERYELLOW,GREEN,RED,WHITE,LOWERYELLOW,BLUE,UPPERVARIABLE,LOWERSTRIP};  //prirazeni pinu pro LED v navestidle
bool ModePredvest=false;  // globalni promena urcujici, zda se bude zobrazovat signal prdvesti, svoji hodnotu ziska pri zapnuti a dale se nemeni
//---------- END LED pin definition
//---------- SCOM signal definition
#define SCOMINPUT 12       //vstupni pin pro signal
#define RECEIVEDETECT 13  //vystupni pin pro signalizaci spravne prijateho SCOM paketu, kratky blik
#define MINTB 4           //min. delka bitu v SCOM prenosu v ms
#define MAXTB 30           //min. delka bitu v SCOM prenosu v ms
#define MINPKTDIST 9      //min. odstup paketu v SCOM prenosu (pocet bitu)
#define MSSAMPLES 5        //pocet samplu za 1ms
#define OKINDIKTIME 5     //delka rozsiceni indik. LED po uspesnem prijmu SCOM paketu v ms  (max hodnota 50ms)
//---------- Setup Mode Code definition
#define SETUPTIMEOUT 2000  // [ms] pokud se v rezimu setup neprijme relevantni kod do tohoto casu, primac se resetuje
#define SETUPMODE 127     //SCOM kod pro vstup do rezimu nastaveni
#define ENDSETUP 124  //ukoncit setup
#define MODHLAV 126 //nastavovani prijimace hlavniho navestidla
#define MODPRED 125 //nastavovani prijimace predvesti (pokud je na stejne lince jako prijimac hlavniho navestidle)
#define SAVE  115     //ulozit
#define SUPPERYELLOW 116 //horni zluta
#define SGREEN 117       //zelena
#define SRED 118         //cervena
#define SWHITE 119       //bila
#define SLOWERYELLOW 120 //dolni zluta
#define SBLUE 121        //modra
#define SUPPERVARIABLE 122 //horni pruh nebo jiny prosvetlovany symbol
#define SLOWERSTRIP 123  //dolni pruh

volatile byte ReceivedSCOM=0; //posledni prijaty platny byte SCOM komunikace,
volatile bool ReceivedFlag=false; // flag se nastavi na true po prijmu SCOM paketu, po precteni se musi shodit. Nenastavuje se po kazdem paketu ale po dvojici paketu ruznych od predchoziho, tedy jen pri zmene dat v signalu.
// ---------- END SCOM signal definition

typedef enum {   //vyctovy typ pro stav svetel navestidla
  LIGHTOFF,LIGHTON,BLINKSLOW,BLINKFAST,BLINKEXTRASLOW
}LEDMODE;
typedef struct {  // struktura pro cely signal na navestidle
  LEDMODE UpperYellow;
  LEDMODE Green;  
  LEDMODE Red;
  LEDMODE White;
  LEDMODE LowerYellow;
  LEDMODE Blue;
  LEDMODE UpperVariable;
  LEDMODE LowerStrip;
}SIGNAL;
volatile LEDMODE Signals[8];  //promenna definujici svetelnou navest, je primo ctena obsluhou preruseni a jeji zmena je provadena funkci SetSignal()

//------------ Fuction prototypes -----------------
ISR(TIMER2_COMPA_vect);  // obslouzeni interruptu od timeru 2 (kazdych 200us) V nem probiha prijem S/Com signalu a PWM modulace osmi vystupu
void SetSignal(SIGNAL DispSignal,volatile LEDMODE IsrSignalArray[8]); // Obsah v promenne DispSignal prevede do promenne se kterou pracuje obsluha interruptu
void GetSignal(byte SCode,SIGNAL *OutSignal); //prevede SCOM kod na navestni signal, v pripade ze nav. kod neni definovan, vrati se signal zhasnuto
void GetSignalPredvest(byte SCode,SIGNAL *OutSignal); //prevede SCOM kod na navestni signal predvesti , v pripade ze nav. kod neni definovan, vrati se signal zhasnuto
bool ReadEeprom(int EeOffset,byte EeVal[8]); //precte a zkontroluje validitu dat v EEProm, pokud je validni, vraci data a true, jinak vraci data a false, zacatek bloku je dan promennou EeOffset
bool WriteEeprom(int EeOffset,byte EeVal[8]); //Zapise data a kontrolni soucet do eeprom a zkontroluje, zda se zapsaly, pokud ano, vraci true, jinak false, zacatek bloku je dan promennou EeOffset
void(* ResetFunc) (void) = 0; //reset funkce pro programovy reset (predani rizeni na adresu 0)   ResetFunc();  //call reset
void ReadAndSetMax(byte MaxInt[8],byte MaxIdx); //cte scom kody a nastavuje podle nich maximalni intensitu MaxInt[MaxIdx]
//------------ END Fuction prototypes -----------------

#include <EEPROM.h>
void setup() {
  pinMode(SCOMINPUT,INPUT_PULLUP);  //nastaveni SCOM vstupu
  pinMode(MODPREDVEST,INPUT_PULLUP);  //nastaveni vstupu pro zapnuti modu predvesti
  pinMode(RECEIVEDETECT,OUTPUT);    //nastaveni vystupu detekce prijateho paketu na interni LED arduina
  digitalWrite(RECEIVEDETECT,LOW);  //zakladni stav detekcni LED je zhasnuto
  
  for (byte i=0;i<8;i++){
    pinMode (OUTPIN[i], OUTPUT);
    digitalWrite(OUTPIN[i],HIGH); //zhasnuto
  };
  for (byte i=0;i<8;i++){
    Signals[i]=LIGHTOFF;               //nastaveni pole pro prime pouziti ISR rutinou na hodnotu zhasnuto pro vsechny signaly 
    MaxIntensity[i]=MaxIntensity[i]/2; //uprava hodnot z % na skutecne kroky podle velikosti PWMSTEPS
    if(MaxIntensity[i]>50) {           //kontrola hodnot maxima
      MaxIntensity[i]=50;
    }
    if(MaxIntensity[i]<3) {           //kontrola hodnot minima (pro plynuly efekt plynuleho nabehu je treba alespon 5 stupnu)
      MaxIntensity[i]=3;
    }
  };
  byte EepromCache[8];
  if(FORCEEEPROMWRITE==1){
    WriteEeprom(EEPROMBASEADR,MaxIntensity);
  }
  if(SETUPENABLE==1){ //kdyz je povolen setup mode, precteme hodnoty z EEPROM, pokud tam jsou ulozene (souhlasi kontrolni soucet vsech hodnot)
   if(ReadEeprom(EEPROMBASEADR,EepromCache)){ //pokud je ulozene platne nastaveni, precte se a ulozi
    for (byte i=0;i<8;i++){
      MaxIntensity[i]=EepromCache[i];
      if(MaxIntensity[i]>50) {           //kontrola hodnot maxima
        MaxIntensity[i]=50;
      }
      if(MaxIntensity[i]<3) {           //kontrola hodnot minima (pro plynuly efekt plynuleho nabehu je treba alespon 5 stupnu)
        MaxIntensity[i]=3;
      }
    }
   }else{// nastaveni neni platne, zapiseme defaultni
    WriteEeprom(EEPROMBASEADR,MaxIntensity);
   }
  }
  if(digitalRead(MODPREDVEST)==LOW){ //neni zapnuty mod predvesti??
    ModePredvest=true;
  }else{
    ModePredvest=false;
  }
  
  // nastaveni timeru 2
  TCCR2A = bit(WGM21);          // nebude vystup na port a rezim bude CTC
  TCCR2B = bit(CS21)|bit(CS20); // zdroj signalu Fosc a preddelic 32, inkrementace kazde 2us
  OCR2A =  99;                  // interrupt a znovunastaveni pri dosazeni stavu, int kazdych 100*2us=200us
  TIMSK2 = bit (OCIE2A);        // povoleni preruseni v masce interruptu
 // END nastaveni timeru 2
}//END of setup()


void loop() {
 SIGNAL Navest;
 byte ScomCode=DEFAULTSIGNAL;
 byte WriteTry=0;
  if(!ModePredvest){
   GetSignal(ScomCode,&Navest);  //nastaveni defaultniho navesti
 }else{
   GetSignalPredvest(ScomCode,&Navest);  //nastaveni defaultniho navesti predvesti
 }
 SetSignal(Navest,Signals);
 do{ // rezim zobrazovani podle prijateho kodu
  if(ReceivedFlag){    //byl prijaty novy kod??
    noInterrupts();
    ScomCode=ReceivedSCOM;  //kod si ulozime
    ReceivedFlag=false;
    interrupts();
    if(ScomCode==SETUPMODE && SETUPENABLE!=1){ //neni povolen rezim setup, tak zmenime kod na zhasnuto (13)
      ScomCode=13;
    }
    if (!ModePredvest){ // podle modu prijimace pouzijeme bud hlavni tabulku navesti nebo tabulku predvesti
      GetSignal(ScomCode,&Navest);  //nastaveni navesti z hlavni tabulky
    }else{
       GetSignalPredvest(ScomCode,&Navest);  //nastaveni predvesti z tabulky predvesti
    }
    SetSignal(Navest,Signals);    
  }  
 }while(ScomCode!=SETUPMODE); //pokud byl prijat kod pro SETUP rezim, koncime normalni zobrazovani a jdem do setupu
 
 //---------- Setup rezim (jeho ukoceni vzdy pres reset)----------------------------------------------------------------------------
 unsigned long DiffTime=millis();
 do{
  if(ReceivedFlag){    //byl prijaty novy kod??
    noInterrupts();
    ScomCode=ReceivedSCOM;  //kod si ulozime
    ReceivedFlag=false;
    interrupts();
  }
  if((millis()-DiffTime)>SETUPTIMEOUT){ //pokud uplynul timeout, resetujeme (ochrana pred nahodnym vyvolanim setupu)
    ResetFunc(); 
  }
 }while(!(ScomCode==MODHLAV || ScomCode==MODPRED));  //cteme kody dokud nedostaneme nastavovaci kod pro predvest nebo hlavni navest
 if((ScomCode==MODHLAV && ModePredvest)||(ScomCode==MODPRED && !ModePredvest)){ //prijaty setup kod neodpovida stavu prijimace
    Navest.UpperYellow=LIGHTOFF;  // nastaveni vsech svetel do stavu zhasnuto
    Navest.Green=LIGHTOFF;
    Navest.Red=LIGHTOFF;
    Navest.White=LIGHTOFF;
    Navest.LowerYellow=LIGHTOFF;
    Navest.Blue=LIGHTOFF;
    Navest.UpperVariable=LIGHTOFF;
    Navest.LowerStrip=LIGHTOFF;
    SetSignal(Navest,Signals); //zhasnuti navesti
    do{                                 //pasivni cekani na detekci konce setupu, pak resetujeme
      if(ReceivedFlag){
        noInterrupts();
        ScomCode=ReceivedSCOM;  //kod si ulozime
        ReceivedFlag=false;
        interrupts();
        if(ScomCode==ENDSETUP){
          delay(2000);  //cekani na odvysilani cele serie paketu ENDSETUP
          ResetFunc();
        }
      }
    }while(true);
 }
 //prijaty setup kod odpovida stavu prijimace
 Navest.UpperYellow=LIGHTON;  // nastaveni vsech svetel do stavu sviti
 Navest.Green=LIGHTON;
 Navest.Red=LIGHTON;
 Navest.White=LIGHTON;
 Navest.LowerYellow=LIGHTON;
 Navest.Blue=LIGHTON;
 Navest.UpperVariable=LIGHTON;
 Navest.LowerStrip=LIGHTON;
 SetSignal(Navest,Signals); //rozsviceni navesti
 do{
  ScomCode=0;
  if(ReceivedFlag){
    noInterrupts();
    ScomCode=ReceivedSCOM;  //kod si ulozime
    ReceivedFlag=false;
    interrupts();
    switch (ScomCode){
      case SUPPERYELLOW:
        Navest.UpperYellow=LIGHTOFF; // na 500ms nastavovane svetlo zhasneme (bliknuti)
        SetSignal(Navest,Signals);
        delay(500);
        Navest.UpperYellow=LIGHTON;
        SetSignal(Navest,Signals);
        ReadAndSetMax(MaxIntensity,0);
      break;
      case SGREEN:
        Navest.Green=LIGHTOFF; // na 500ms nastavovane svetlo zhasneme (bliknuti)
        SetSignal(Navest,Signals);
        delay(500);
        Navest.Green=LIGHTON;
        SetSignal(Navest,Signals);
        ReadAndSetMax(MaxIntensity,1);
      break;
      case SRED:
        Navest.Red=LIGHTOFF; // na 500ms nastavovane svetlo zhasneme (bliknuti)
        SetSignal(Navest,Signals);
        delay(500);
        Navest.Red=LIGHTON;
        SetSignal(Navest,Signals);
        ReadAndSetMax(MaxIntensity,2);
      break;
      case SWHITE:
        Navest.White=LIGHTOFF; // na 500ms nastavovane svetlo zhasneme (bliknuti)
        SetSignal(Navest,Signals);
        delay(500);
        Navest.White=LIGHTON;
        SetSignal(Navest,Signals);
        ReadAndSetMax(MaxIntensity,3);
      break;
      case SLOWERYELLOW:
        Navest.LowerYellow=LIGHTOFF; // na 500ms nastavovane svetlo zhasneme (bliknuti)
        SetSignal(Navest,Signals);
        delay(500);
        Navest.LowerYellow=LIGHTON;
        SetSignal(Navest,Signals);
        ReadAndSetMax(MaxIntensity,4);
      break;
      case SBLUE:
        Navest.Blue=LIGHTOFF; // na 500ms nastavovane svetlo zhasneme (bliknuti)
        SetSignal(Navest,Signals);
        delay(500);
        Navest.Blue=LIGHTON;
        SetSignal(Navest,Signals);
        ReadAndSetMax(MaxIntensity,5);
      break;
      case SUPPERVARIABLE:
        Navest.UpperVariable=LIGHTOFF; // na 500ms nastavovane svetlo zhasneme (bliknuti)
        SetSignal(Navest,Signals);
        delay(500);
        Navest.UpperVariable=LIGHTON;
        SetSignal(Navest,Signals);
        ReadAndSetMax(MaxIntensity,6);
      break;
      case SLOWERSTRIP:
        Navest.LowerStrip=LIGHTOFF; // na 500ms nastavovane svetlo zhasneme (bliknuti)
        SetSignal(Navest,Signals);
        delay(500);
        Navest.LowerStrip=LIGHTON;
        SetSignal(Navest,Signals);
        ReadAndSetMax(MaxIntensity,7);
      break;
      case SAVE:
        WriteTry=0;
        while((!(WriteEeprom(EEPROMBASEADR,MaxIntensity)))&&(WriteTry<3)){ //tri pokusy zapsani do EEPROM
          WriteTry++; 
        }
        if(WriteTry>=3){ //nepovedlo se zapsat ani na tri pokusy, nechame vsechna svetla blikat velmi pomalu
           Navest.UpperYellow=BLINKEXTRASLOW;  // nastaveni vsech svetel do stavu blika velmi pomalu
           Navest.Green=BLINKEXTRASLOW;
           Navest.Red=BLINKEXTRASLOW;
           Navest.White=BLINKEXTRASLOW;
           Navest.LowerYellow=BLINKEXTRASLOW;
           Navest.Blue=BLINKEXTRASLOW;
           Navest.UpperVariable=BLINKEXTRASLOW;
           Navest.LowerStrip=BLINKEXTRASLOW;
           SetSignal(Navest,Signals); //rozsviceni navesti
           do{
            ;
           }while(true); //jedina moznost ukonceni tohoto chyboveho stavu je reset nebo PWR OFF/ON          
        }else{ //zapis do EEPROM je OK, jedenkrat blikneme vsemi svetly
           Navest.UpperYellow=LIGHTOFF;  // nastaveni vsech svetel do stavu zhasnuto
           Navest.Green=LIGHTOFF;
           Navest.Red=LIGHTOFF;
           Navest.White=LIGHTOFF;
           Navest.LowerYellow=LIGHTOFF;
           Navest.Blue=LIGHTOFF;
           Navest.UpperVariable=LIGHTOFF;
           Navest.LowerStrip=LIGHTOFF;
           SetSignal(Navest,Signals); //zhasnuti navesti
           delay(500);
           Navest.UpperYellow=LIGHTON;  // nastaveni vsech svetel do stavu sviti
           Navest.Green=LIGHTON;
           Navest.Red=LIGHTON;
           Navest.White=LIGHTON;
           Navest.LowerYellow=LIGHTON;
           Navest.Blue=LIGHTON;
           Navest.UpperVariable=LIGHTON;
           Navest.LowerStrip=LIGHTON;
           SetSignal(Navest,Signals); //rozsviceni navesti          
        }
      break;
      default:
      break;    
      }
    }
  }while(!(ScomCode==ENDSETUP)); //kod  ENDSETUP ukonci nastavovani svetel
  delay(2000);//cekani na odvysilani cele serie paketu ENDSETUP
  ResetFunc();
 //------ END Setup rezim ------------------------------------------------------------------------------------
  
} // END of loop

void GetSignal(byte SCode,SIGNAL *OutSignal){ //prevede SCOM kod na navestni signal, v pripade ze nav. kod neni definovan, vrati se signal zhasnuto
  OutSignal->UpperYellow=LIGHTOFF;  // nastaveni vsech svetel do stavu zhasnuto
  OutSignal->Green=LIGHTOFF;
  OutSignal->Red=LIGHTOFF;
  OutSignal->White=LIGHTOFF;
  OutSignal->LowerYellow=LIGHTOFF;
  OutSignal->Blue=LIGHTOFF;
  OutSignal->UpperVariable=LIGHTOFF;
  OutSignal->LowerStrip=LIGHTOFF;
  switch (SCode){
    case 0://stuj,posun zakazan
      OutSignal->Red=LIGHTON;
      OutSignal->Blue=LIGHTON;
    break;
    case 1://volno
      OutSignal->Green=LIGHTON;
    break;
    case 2://vystraha
      OutSignal->UpperYellow=LIGHTON;
    break;
    case 3://ocekavej 40
      OutSignal->UpperYellow=BLINKSLOW;
    break;
    case 4://rychlost 40 a volno
      OutSignal->Green=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
    break;
    case 5://test vsech svetel
      OutSignal->UpperYellow=LIGHTON;  // nastaveni vsech svetel do stavu sviti
      OutSignal->Green=LIGHTON;
      OutSignal->Red=LIGHTON;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->Blue=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
      OutSignal->LowerStrip=LIGHTON;
    break;
    case 6://rychlost 40 a vystraha
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
    break;
    case 7://rychlost 40 a ocekavej 40
      OutSignal->UpperYellow=BLINKSLOW;
      OutSignal->LowerYellow=LIGHTON;
    break;
    case 8://opatrne na privolavaci navest s cervenou
      OutSignal->Red=LIGHTON;
      OutSignal->White=BLINKSLOW;
    break;
    case 9://posun povolen
      OutSignal->White=LIGHTON;
    break;
    case 10://nezabezpeceny posun povolen
      OutSignal->Red=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 11://opakovana volno
      OutSignal->Green=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 12://opakovana vystraha
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 13://zhasnuto
    break;
    case 14://opakovana ocekavej 40
      OutSignal->UpperYellow=BLINKSLOW;
      OutSignal->White=LIGHTON;
    break;
    case 15://rychlost 40 a opakovana vystraha
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
    break;
    case 16://rychlost 40 a opakovana ocekavej 40 
      OutSignal->UpperYellow=BLINKSLOW;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
    break;
    case 17://jizda podle rozhledovych pomeru
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->White=BLINKSLOW;
    break;
    case 18://rychlost 40 a jizda podle rozhledovych pomeru
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->White=BLINKSLOW;
      OutSignal->LowerYellow=LIGHTON;
    break;
    case 19://opatrne na privolavaci navest bez cervene
      OutSignal->White=BLINKSLOW;
    break;
    case 30://ocekavej 60 
      OutSignal->UpperYellow=BLINKFAST;
    break;
    case 31://ocekavej 80 
      OutSignal->Green=BLINKSLOW;
    break;
    case 32://ocekavej 100 
      OutSignal->Green=BLINKFAST;
    break;
    case 33://rychlost 40 a ocekavej 60 
      OutSignal->UpperYellow=BLINKFAST;
      OutSignal->LowerYellow=LIGHTON;
    break;
    case 34://rychlost 40 a ocekavej 80 
      OutSignal->Green=BLINKSLOW;
      OutSignal->LowerYellow=LIGHTON;
    break;
    case 35://rychlost 40 a ocekavej 100 
      OutSignal->Green=BLINKFAST;
      OutSignal->LowerYellow=LIGHTON;
    break;
    case 36://rychlost 60 a volno 
      OutSignal->Green=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 37://rychlost 60 a vystraha 
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 38://rychlost 60 a ocekavej 40 
      OutSignal->UpperYellow=BLINKSLOW;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 39://rychlost 60 a ocekavej 60
      OutSignal->UpperYellow=BLINKFAST;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 40://rychlost 60 a ocekavej 80
      OutSignal->Green=BLINKSLOW;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 41://rychlost 60 a ocekavej 100
      OutSignal->Green=BLINKFAST;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 42://rychlost 80 a volno 
      OutSignal->Green=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 43://rychlost 80 a vystraha 
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 44://rychlost 80 a ocekavej 40 
      OutSignal->UpperYellow=BLINKSLOW;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 45://rychlost 80 a ocekavej 60
      OutSignal->UpperYellow=BLINKFAST;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 46://rychlost 80 a ocekavej 80 
      OutSignal->Green=BLINKSLOW;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 47://rychlost 80 a ocekavej 100
      OutSignal->Green=BLINKFAST;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 48://rychlost 100 a volno 
      OutSignal->Green=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
      OutSignal->LowerStrip=LIGHTON;
    break;
    case 49://rychlost 100 a vystraha 
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
      OutSignal->LowerStrip=LIGHTON;
    break;
    case 50://rychlost 100 a ocekavej 40 
      OutSignal->UpperYellow=BLINKSLOW;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
      OutSignal->LowerStrip=LIGHTON;
    break;
    case 51://rychlost 100 a ocekavej 60
      OutSignal->UpperYellow=BLINKFAST;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
      OutSignal->LowerStrip=LIGHTON;
    break;
    case 52://rychlost 100 a ocekavej 80 
      OutSignal->Green=BLINKSLOW;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
      OutSignal->LowerStrip=LIGHTON;
    break;
    case 53://rychlost 100 a ocekavej 100
      OutSignal->Green=BLINKFAST;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
      OutSignal->LowerStrip=LIGHTON;
    break;
     case 60://opakovana ocekavej 60 
      OutSignal->UpperYellow=BLINKFAST;
      OutSignal->White=LIGHTON;
    break;
    case 61://opakovana ocekavej 80 
      OutSignal->Green=BLINKSLOW;
      OutSignal->White=LIGHTON;
    break;
    case 62://opakovana ocekavej 100 
      OutSignal->Green=BLINKFAST;
      OutSignal->White=LIGHTON;
    break;
    case 63://rychlost 40 a opakovana volno
      OutSignal->Green=LIGHTON;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
    break;
    case 64://rychlost 40 a opakovana ocekavej 60
      OutSignal->UpperYellow=BLINKFAST;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
    break;
    case 65://rychlost 40 a opakovana ocekavej 80
      OutSignal->Green=BLINKSLOW;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
    break;
    case 66://rychlost 40 a opakovana ocekavej 100
      OutSignal->Green=BLINKFAST;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
    break;
   case 67://rychlost 60 a opakovana volno 
      OutSignal->Green=LIGHTON;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 68://rychlost 60 a opakovana vystraha 
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 69://rychlost 60 a opakovana ocekavej 40 
      OutSignal->UpperYellow=BLINKSLOW;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 70://rychlost 60 a opakovana ocekavej 60
      OutSignal->UpperYellow=BLINKFAST;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 71://rychlost 60 a opakovana ocekavej 80
      OutSignal->Green=BLINKSLOW;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 72://rychlost 60 a opakovana ocekavej 100
      OutSignal->Green=BLINKFAST;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 73://rychlost 80 a opakovana volno 
      OutSignal->Green=LIGHTON;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 74://rychlost 80 a opakovana vystraha 
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 75://rychlost 80 a opakovana ocekavej 40 
      OutSignal->UpperYellow=BLINKSLOW;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 76://rychlost 80 a opakovana ocekavej 60
      OutSignal->UpperYellow=BLINKFAST;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 77://rychlost 80 a opakovana ocekavej 80 
      OutSignal->Green=BLINKSLOW;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 78://rychlost 80 a opakovana ocekavej 100
      OutSignal->Green=BLINKFAST;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 90://vlozena stuj
      OutSignal->Red=LIGHTON;
    break;
    case 91://vlozena stuj s modrou
      OutSignal->Red=LIGHTON;
      OutSignal->Blue=LIGHTON;
    break;
    case 92://vlozena odjezdove navestidlo povoluje jizdu
      OutSignal->White=BLINKEXTRASLOW;  // pozdejsi navestni predpisy uvadeji BLINKSLOW
    break;
    case 93://vlozena posun zakazan
      OutSignal->Blue=LIGHTON;
    break;
    case 94://vlozena posun povolen
      OutSignal->White=LIGHTON;
    break;
    case 95://vlozena posun povolen-nezabezpeceny
      OutSignal->Red=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 100://spadovistni sunout zakazano
      OutSignal->Red=LIGHTON;
    break;
    case 101://spadovistni sunout pomalu
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 102://spadovistni sunout rychleji
      OutSignal->UpperYellow=LIGHTON;
    break;
    case 103://spadovistni sunout zpet
      OutSignal->Red=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
    case 104://spadovistni opakovana sunout zakazano
      OutSignal->Blue=LIGHTON;
    break;
    case 105://spadovistni opakovana sunout pomalu
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 106://spadovistni opakovana sunout rychleji
      OutSignal->UpperYellow=LIGHTON;
    break;
    case 107://spadovistni opakovana sunout zpet
      OutSignal->Blue=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
    break;
   default:
    break;
  } 
}

void GetSignalPredvest(byte SCode,SIGNAL *OutSignal){ //prevede SCOM kod na navestni signal predvesti , v pripade ze nav. kod neni definovan, vrati se signal zhasnuto
  OutSignal->UpperYellow=LIGHTOFF;  // nastaveni vsech svetel do stavu zhasnuto
  OutSignal->Green=LIGHTOFF;
  OutSignal->Red=LIGHTOFF;
  OutSignal->White=LIGHTOFF;
  OutSignal->LowerYellow=LIGHTOFF;
  OutSignal->Blue=LIGHTOFF;
  OutSignal->UpperVariable=LIGHTOFF;
  OutSignal->LowerStrip=LIGHTOFF;
  switch (SCode){
    case 0:
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 1:
      OutSignal->Green=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 2:
      OutSignal->Green=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 3:
      OutSignal->Green=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 4:
      OutSignal->UpperYellow=BLINKSLOW;
      OutSignal->White=LIGHTON;
    break;
    case 5://test vsech svetel
      OutSignal->UpperYellow=LIGHTON;  // nastaveni vsech svetel do stavu sviti
      OutSignal->Green=LIGHTON;
      OutSignal->Red=LIGHTON;
      OutSignal->White=LIGHTON;
      OutSignal->LowerYellow=LIGHTON;
      OutSignal->Blue=LIGHTON;
      OutSignal->UpperVariable=LIGHTON;
      OutSignal->LowerStrip=LIGHTON;
    break;
    case 6:
      OutSignal->UpperYellow=BLINKSLOW;
      OutSignal->White=LIGHTON;
    break;
    case 7:
      OutSignal->UpperYellow=BLINKSLOW;
      OutSignal->White=LIGHTON;
    break;
    case 8:
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 9:
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 10:
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 11:
      OutSignal->Green=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 12:
      OutSignal->UpperYellow=LIGHTON;
      OutSignal->White=LIGHTON;
    break;
    case 13:
    break;
    case 14:
      OutSignal->UpperYellow=BLINKSLOW;
    break;
    case 15:
      OutSignal->UpperYellow=LIGHTON;
    break;
   default:
    break;
  } 
}

bool ReadEeprom(int EeOffset,byte EeVal[8]){ //precte a zkontroluje validitu dat v EEProm, pokud je validni, vraci data a true, jinak vraci data a false, zacatek bloku je dan promennou EeOffset
  int Sum=0;
  for(byte i=0;i<8;i++){
    EeVal[i]=EEPROM.read(EeOffset+i);
    Sum=Sum+EeVal[i];
  }
  if(Sum>0 && highByte(Sum)==EEPROM.read(EeOffset+8) && lowByte(Sum)==EEPROM.read(EeOffset+9)){ //kontrola souctu, ktery musi byt zaroven nenulovy
    return true;
  }else{
    return false;
  }
}

bool WriteEeprom(int EeOffset,byte EeVal[8]){ //Zapise data a kontrolni soucet do eeprom a zkontroluje, zda se zapsaly, pokud ano, vraci true, jinak false
  int Sum=0;
  for(byte i=0;i<8;i++){
    EEPROM.update(EeOffset+i,EeVal[i]);
    if(EEPROM.read(EeOffset+i)!=EeVal[i]){ //kontrola zapisu prectenim
      return false;
    }
    Sum=Sum+EeVal[i];
  }
  EEPROM.update(EeOffset+8,highByte(Sum)); // zapis dvou byte kontrolniho souctu
  EEPROM.update(EeOffset+9,lowByte(Sum));
  if(highByte(Sum)==EEPROM.read(EeOffset+8) && lowByte(Sum)==EEPROM.read(EeOffset+9)){ //kontrola zapsani kontrolniho souctu
    return true;
  }else{
    return false;
  }
}


void SetSignal(SIGNAL DispSignal,volatile LEDMODE IsrSignalArray[8]){ // Obsah v promenne DispSignal prevede do promenne se kterou pracuje obsluha interruptu
  noInterrupts();
  IsrSignalArray[0]=DispSignal.UpperYellow;
  IsrSignalArray[1]=DispSignal.Green;
  IsrSignalArray[2]=DispSignal.Red;
  IsrSignalArray[3]=DispSignal.White;
  IsrSignalArray[4]=DispSignal.LowerYellow;
  IsrSignalArray[5]=DispSignal.Blue;
  IsrSignalArray[6]=DispSignal.UpperVariable;
  IsrSignalArray[7]=DispSignal.LowerStrip;
  interrupts();
}

void ReadAndSetMax(byte MaxInt[8],byte MaxIdx){ //cte scom kody a nastavuje podle nich maximalni intensitu MaxInt[MaxIdx]
 byte SCode;
 do{
  SCode=0;
  if(ReceivedFlag){
    noInterrupts();
    SCode=ReceivedSCOM;  //kod si ulozime
    ReceivedFlag=false;
    interrupts();
  }
 if(SCode>=6 && SCode<=100){
  MaxInt[MaxIdx]=SCode/2; 
 }
 }while(SCode!=SETUPMODE); //opakuje se tak dlouho, dokud neni prijat kod SETUPMODE, ktery ukoncuje nastavovani svetla
}


ISR(TIMER2_COMPA_vect)  // obslouzeni interruptu od timeru 2 po dosazeni OCR2A (kazdych 200us)
{ 
  // promenne pro PWM a zobrazeni navesti
  static byte PWMCounter=0;
  static byte DelayCounter=0;
  static byte PrescalerA=0;
  static byte PrescalerB=0;
  static byte BlinkTimer=0;
  static byte IncRegister[8]={0,0,0,0,0,0,0,0};
  static byte DelayPrescaler[8]={0,0,0,0,0,0,0,0};
  static byte NewIntensity[8]={0,0,0,0,0,0,0,0};
  static byte ActualIntensity[8]={0,0,0,0,0,0,0,0};   //okamzita hodnota jasu LED
  static bool ChangeFlag=false;  //pri zmene svetel true
  // promenne pro prijem SCOM
  static byte StartBitLen=MAXTB*MSSAMPLES;  //delka Startbitu v nasobcich 200us
  static byte LowLevelLen=0;  //mereni delky aktualni Low urovne SCOM signalu
  static int  HighLevelLen=0;  //mereni delky aktualni High urovne SCOM signalu
  static int  HighLevelLenPrev=0;
  static int  PacketDistance=MINPKTDIST*MAXTB*5; //delka mezi bloky (Tp) SCOM signalu v nasobcich 200us
  static byte SCOMByte=0; //ukladani prave zpracovavaneho bytu SCOM signalu
  static byte SCOMPrev[2]={255,254}; //naposledy spravne prijate byte SCOM signalu pro porovnani shody naslednych paketu, [0]-predpredchozi, [1]-predchozi
  static byte SCOMShiftReg=1;
  static byte SCOMTimeCounter=0;
  static byte SCOMBitCounter=0;  //pocitadlo v nasobcich 200us pro vzorkovani SCOM signalu
  static bool ReceiveFlag=false; // nastaven na true po cas prijimani paketu
  static bool StartOkFlag=false; // nastaven na true po spravnem prijeti startbitu (0,1)
  static byte SCOMLevel=0;        //aktualni uroven SCOM
  static byte SCOMLevelPrev=0xAA;    //minula uroven SCOM
  static byte SCOMOkTimer=0;  //mereni doby indikace spravne prijateho bloku 
  //******* blok cteni SCOM signalu *******
  SCOMLevel=digitalRead(SCOMINPUT);
  if (SCOMLevelPrev==0xAA){     //v prvni periode cteni se musi nastavit minula hodnota shodne s aktualni
    SCOMLevelPrev=SCOMLevel;
  }
  if (SCOMLevelPrev!=SCOMLevel){
    if(SCOMLevel==HIGH){  //zmena z L->H, musile osetrit pocitadlo low
      if(((LowLevelLen>=(MINTB*MSSAMPLES)-1)&&(LowLevelLen<=(MAXTB*MSSAMPLES)+1))){   // byla uroven v rozmezi startbitu SCOM?
        if(!ReceiveFlag){   //mohl by to byt startbit, protoze ma spravnou delku a prijimani neprobiha
          if(HighLevelLenPrev>=(int)(LowLevelLen*MINPKTDIST)){  //byla predchozi uroven tak dlouho, aby to byla mezipaketova mezera?
            ReceiveFlag=true; //nastavime prijem
            StartBitLen=LowLevelLen; //zachovame delku startbitu
            SCOMByte=0; 
            SCOMShiftReg=1;
            SCOMBitCounter=0;
            SCOMTimeCounter=0;
            StartOkFlag=false;
          }
        }
      }
     LowLevelLen=0; //vvynulovat pocitadlo
    }else{ //zmena z H->L, musile osetrit pocitadlo high
      HighLevelLenPrev=HighLevelLen; //zapamatujeme si posledni hodnotu
      HighLevelLen=0; //vynulujeme aktualni pocitadlo
    }
  }
  if(SCOMLevel==HIGH){ //pocitame delku H
    if(HighLevelLen<=(MAXTB*MINPKTDIST*MSSAMPLES)+MSSAMPLES){
      HighLevelLen++;
    }
  }else{  //pocitame delku L
    if(LowLevelLen<(MAXTB*MSSAMPLES)+MSSAMPLES){
      LowLevelLen++;
    }
  }
  if(ReceiveFlag){  //probiha prijem paketu SCOM, zacina se od vzestupne hrany synchronizacniho L bitu
    SCOMTimeCounter++;
    if(StartOkFlag){  //byly uz korektne prijaty startbity 0,1 ?
      if(SCOMTimeCounter==StartBitLen){ //cas na vzorkovani v polovine bitu
        SCOMTimeCounter=0;
        if(SCOMBitCounter<=7){ //prijmeme 7bitu dat a stopbit jako osmy bit dat
          if(SCOMLevel==HIGH){
            SCOMByte=SCOMByte+SCOMShiftReg; //je li jednickovy byt, zvetsime hodnotu dat        
          }
          SCOMShiftReg=SCOMShiftReg<<1;          
        }
       if(SCOMBitCounter==8){ //osmy bit prenosu musi byt jednickovy, je to zacatek mezery mezi bloky
         if((SCOMByte<128)&&(SCOMLevel==HIGH)){ //pokud je bit7=0 a bit8=1, tak je prenos OK
          SCOMOkTimer=1;                  //nastaveni indikacniho casovace na nenulovou uroven (zacne citat)
          digitalWrite(RECEIVEDETECT,HIGH);  //rozviceni LED indikace uspesne prijateho paketu
          if(SCOMByte!=SCOMPrev[1]){ //jiny byte nez minule
            SCOMPrev[0]=SCOMPrev[1];  //posoupneme minulou hodnotu na predminulou
            SCOMPrev[1]=SCOMByte;   //na pozici minulou dame soucasny byte
          }else{ //stejny byte co minule
            if(SCOMPrev[0]!=SCOMPrev[1]){ //soucasny a minuly byte stejne, predminuly je jiny, mus9me nastavit flag prijateho byte
              ReceivedSCOM=SCOMByte;  //prijata hodnota do globalni promene pro cteni mimo interrupt
              ReceivedFlag=true;      //nastaven globalni flag prijmu, po precteni hodnoty z hlavniho programu ho musi shodit
              SCOMPrev[0]=SCOMPrev[1];  //posoupneme minulou hodnotu na predminulou
              SCOMPrev[1]=SCOMByte;   //na pozici minulou dame soucasny byte              
            }            
          }
         }
         ReceiveFlag=false;   //ukonceni cteni bloku
       }
       SCOMBitCounter++;
      }
          
    }else{  //jeste se prijimaji start bity 0,1
      if(SCOMTimeCounter==StartBitLen/2){ //cas pro vzorkovani startbitu 1 v polovine
        if(SCOMLevel==HIGH){ //je to uroven H, prijali jsme startbit 1
          StartOkFlag=true;
          SCOMTimeCounter=0;  //snulovan v polovine bitu, od ted budeme citat do hodnoty StartBitLen, tak to bude v polovine datovych bitu
        }else{  //jina uroven nez H, neni to druhy startbit, rusime prijem
         ReceiveFlag=false; 
        }
      }
    }
  }
  if(SCOMOkTimer>0){
    SCOMOkTimer++;
    if(SCOMOkTimer>(OKINDIKTIME*MSSAMPLES)){    //konec indikace?
      digitalWrite(RECEIVEDETECT,LOW);  //zhasnuti LED indikace uspesne prijateho paketu 
      SCOMOkTimer=0; //deaktivace timeru
    }
  }
  SCOMLevelPrev=SCOMLevel;
  //******* KONEC **** blok cteni SCOM signalu *******
  //******* blok indikace LED *******
  if (++PrescalerA==39){
    PrescalerA=0;
    ++BlinkTimer;    //inkrementace kazdych cca 8ms
                  // b7   b6  b5  b4  b3 b2 b1 b0 [bit]
                  //1024 512 256 128  64 32 16  8 [ms]
   }
  //detekce zmen svetelneho signalu
  if(!ChangeFlag){ //nova zmena pouze po dokonceni predchozi
    for(byte i=0;i<8;i++){
      switch (Signals[i]) {
        case LIGHTOFF: //zhasnuto
          NewIntensity[i]=0;
          break;
        case LIGHTON: //sviti trvale
          NewIntensity[i]=MaxIntensity[i];
          break;
        case BLINKSLOW: //blika pomalu
          if ((BlinkTimer & bit(6))>0){
            NewIntensity[i]=MaxIntensity[i];
          }else{
            NewIntensity[i]=0;
          }
          break;
        case BLINKEXTRASLOW: //blika velmi pomalu
          if ((BlinkTimer & bit(7))>0){
            NewIntensity[i]=MaxIntensity[i];
          }else{
            NewIntensity[i]=0;
          }
          break;
        case BLINKFAST: //blika rychle
          if ((BlinkTimer & bit(5))>0){
            NewIntensity[i]=MaxIntensity[i];
          }else{
            NewIntensity[i]=0;
          }
          break;
      }
      DelayPrescaler[i]=0;
      if(NewIntensity[i]!=ActualIntensity[i]){
        ChangeFlag=true;
        DelayCounter=0;
        if(NewIntensity[i]>ActualIntensity[i]){
          IncRegister[i]=((byte)DELAY)/(NewIntensity[i]-ActualIntensity[i]); //musi byt kladny vysledek
        }else{
          IncRegister[i]=((byte)DELAY)/(ActualIntensity[i]-NewIntensity[i]);
        }
      }
    }
  }
  //Konec*** detekce zmen svetelneho signalu
  //zmena intensity jasu podle nastaveneneho zpozdeni DELAY
  PrescalerB++;
  if (ChangeFlag &&(PrescalerB==4)){  //probehne kazdou milisekundu pri nastavenem flagu zmeny
    PrescalerB=0;
    DelayCounter++;  // citac prodlevy v ms
    for(byte i=0;i<8;i++){
      if(NewIntensity[i]!=ActualIntensity[i]){ // zmena jasu jen pokud se lisi nova hodnota od aktualni
        DelayPrescaler[i]++;
        if(IncRegister[i]==DelayPrescaler[i]){ //je dosazen stav pro inkrementaci??
          DelayPrescaler[i]=0;
          if(NewIntensity[i]>ActualIntensity[i]){ //intenzita se bude zvysovat
            ActualIntensity[i]++; //zvysit intenzitu
            if (NewIntensity[i]<ActualIntensity[i]){ // ohlidat preteceni
              ActualIntensity[i]=NewIntensity[i];
            }
          }else{ //intenzita se bude snizovat
            if(ActualIntensity[i]>0){ //ohlidat podteceni
              ActualIntensity[i]--; //snizit Intensitu
            }
          }
        }
      }
    }            
    if(DelayCounter>=DELAY){
      ChangeFlag=false;
      PrescalerB=0;
      DelayCounter=0;
      for(byte i=0;i<8;i++){
        ActualIntensity[i]=NewIntensity[i];
      }
    }
  }
  //Konec*** zmena intensity jasu podle nastaveneneho zpozdeni DELAY
  // PWM cyklus start; Modulace osmi kanalu podle ActualIntensity[x]
  PWMCounter++;
  if (PWMCounter>=PWMSTEPS){ //PWMSTEPS*200us
    PWMCounter=0;
  }
  for (byte i=0; i<8; i++){
    if (ActualIntensity[i]>PWMCounter){
      digitalWrite(OUTPIN[i],LOW);  //rozsviceno 
    }else{
      digitalWrite(OUTPIN[i],HIGH); //zhasnuto    
    }
  }
   // PWM cyklus stop
 //******* KONEC **** blok indikace LED ******* 
}//end of ISR
