/*
﻿###################################################################################################################################################################
#                               Software for jakekoli Arduino (Uno/Nano,Leonardo,Mega,Pro Mini) s min. 1KB RAM a 5V IN/OUT                                        #
#     SW pro nastavovani modulu S-COM rozsireneho prijimace a navestniho dekoderu                                                                                 #
#     Ver:1.0.                                                                                                                                                    #
#     august 2019                                                                                                                                                 #
#     autor:Ondrej Nadrchal email: onadr@seznam.cz                                                                                                                #  
#     Licenced: GNU                                                                                                                                               #
###################################################################################################################################################################
Tento program slouzi pro nastavvovani modulu S-COM rozsireneho prijimace a navestniho dekoderu po S/COM lince a generovani libovolnych S/COM kodu pro testovaci ucely.
Program nahrajte do libovolneho Arduina (min. 1KB RAM a 5V vstupy/vystupy). Vystup signalu (13) zapojime na S/COM vstup nastavovaneho nebo testovaneho prijimace/dekoderu.
  Pozn. pri pouziti procesoru s 1KB RAM generuje prekladac varovani o nedostatku pameti, ale program bezchybne funguje (overeno!).
Spustte Seriovy monitor na prislusnem COMu a ridte se zobrazovanymi instrukcemi.
Je mozne odddelene nastavovat moduly na jedne S-COM lince, pokud je jeden modul v modu predvesti a druhy v modu hlavni navesti.
Je mozne najednou nastavovat vice modulu ve stejnem modu na jedne S-COM lince (paralelni zapojeni).
Je mozne generovat libovolny S-COM kod pro testovani dekoderu s navestmi.
Pro testovani jinych S-COM dekoderu nastavte hodnotu ADDPACKETDISTANCE tak, aby mezi pakety bylo zpozdeni min. 200ms. (Tento program generuje mezeru 10xTb + ADDPACKETDISTANCE [ms])
*/
// program pro Nastaveni SCOM prijimace,komunikace a volby po seriove lince pres seriovy monitor
#define BITLENGHT 15  //delka bitu v ms
#define REPEATPACKET 4 //pocet opakovani paketu
#define SCOMOUTPUT 13   //scom vystup
#define ADDPACKETDISTANCE 0 //pridane zpozdeni mezi pakety v ms (vysledne zpozdeni je (10xTb) + ADDPACKETDISTANCE)
//definice ridicich kodu pro nastaveni
#define SETUP 127 //vstup do setupu
#define ENDSETUP 124  //ukoncit setup
#define MODHLAV 126 //nastavovani prijimace hlavniho navestidla
#define MODPRED 125 //nastavovani prijimace predvesti (pokud je na stejne lince jako prijimac hlavniho navestidle)
#define SAVE  115     //ulozit
#define UPPERYELLOW 116 //horni zluta
#define GREEN 117       //zelena
#define RED 118         //cervena
#define WHITE 119       //bila
#define LOWERYELLOW 120 //dolni zluta
#define BLUE 121        //modra
#define UPPERVARIABLE 122 //horni pruh nebo jiny prosvetlovany symbol
#define LOWERSTRIP 123  //dolni pruh
const char InputError[]={"Chybne zadani!"};
const char NumSelection[]={"Zadejte cislo volby a Enter."};
byte Tb=BITLENGHT;

void setup() {
  // put your setup code here, to run once:
  pinMode(SCOMOUTPUT,OUTPUT);
  Serial.begin(9600);
  while (!Serial) {
    ;//cekani na aktivaci serioveho portu
  }
  digitalWrite(SCOMOUTPUT,HIGH);
  delay(BITLENGHT*10);

}
//-------- FUNCTION protorypes -------
void SCOMTransmit (byte Data,byte BitLen,byte RepeatTransmit);   //opakovane vysle zadany bit SCOM protokolem s definovanou delkou bitu v ms, vcetne meziblokove mezery
void SetLight(byte LCode); //nastavi jas svetla v prijimaci
char ReadChar(); //cte ze seriove linky dokud nenarazi na prvni pismeno nebo cislici, pak docte do eol a to zahodi, vraci velkou hodnotu pismena nebo cislici
int ReadNum(); //precte cislo ze serioveho portu v rozsahu 0-999, max tri cislice bez znamenka, dalsi se budou ignorovat, pri neplatnem cteni vraci -1
void PrintSeparatorLine(char OutChar,byte CharNum);// posle na seriovou konzoli znak OutChar v poctu CharNum a odradkuje
//-------- END FUNCTION protorypes -------


void loop() {
  char MSelection;
  Serial.println("S/COM prijimac konfigurator . Verze 1.0");
  do{    
      PrintSeparatorLine('-',65);
      Serial.println("1.Setup hlavniho navestidla.");
      Serial.println("2.Setup predvesti.");
      Serial.println("3.Generator kodu.");
      Serial.print("4.Konfigurace S-COM. Delka bitu = ");
      Serial.print(Tb);
      Serial.println(" ms.");
      Serial.println(NumSelection);
      do{
        MSelection=ReadChar();
      }while(MSelection<'1' || MSelection >'4');
      PrintSeparatorLine('-',65);  
//------------------------------------------------------------------------------------------------------------------ 
      if(MSelection=='1'){  //setup hlavniho navestidla
        Serial.println("HLAVNI NAVEST.");
        Serial.println("Na nastav. navesti se rozsviti vse, pripojene predvesti zhasnou.");  
        SCOMTransmit(SETUP,Tb,REPEATPACKET);  //jdi do setupu                
        SCOMTransmit(MODHLAV,Tb,REPEATPACKET); // mod hlavni navesti
        MSelection='P'; //pokracovat se bude v sekci S, ktera je spolecna pro predvest i hlavni navestidlo
      }
//------------------------------------------------------------------------------------------------------------------   
      if(MSelection=='2'){  //setup predvesti
        Serial.println("PREDVEST.");
        Serial.println("Na nastav. predvesti se rozsviti vse, pripojene hl.navesti zhasnou.");  
        SCOMTransmit(SETUP,Tb,REPEATPACKET);  //jdi do setupu                
        SCOMTransmit(MODPRED,Tb,REPEATPACKET); // mod pedvesti
        MSelection='P'; //pokracovat se bude v sekci S, ktera je spolecna pro predvest i hlavni navestidlo
        
      }
//------------------------------------------------------------------------------------------------------------------     
      if(MSelection=='3'){ //generovani scom kodu
        int Code;
        do{
          Serial.println("Zadej kod 0-126 (>126 pro exit) ");
          do{
            Code=ReadNum();
            if(Code<0){
              Serial.println(InputError);
            }
          }while(Code<0);        
          Serial.println(Code);   
          if(Code<127){
          SCOMTransmit(lowByte(Code),Tb,REPEATPACKET);    
          }
        }while(Code<127);
      }
//------------------------------------------------------------------------------------------------------------------     
      if(MSelection=='4'){  //konfigurace scomu
        Serial.println("Zadej Tb (4-30ms)");
        int TbVal;
        do{
          TbVal=ReadNum();
          if(TbVal<4 || TbVal>30){
            Serial.println(InputError);
          }
        }while(TbVal<4 || TbVal>30);
        Tb=TbVal;
        Serial.println();
      }
//------------------------------------------------------------------------------------------------------------------     
      if(MSelection=='P'){  //setup hlav.navesti i predvesti, pokracovani volby 1 nebo 2        
        PrintSeparatorLine('-',65); 
        do{
          Serial.println("1.Horni zluta");
          Serial.println("2.Zelena");
          Serial.println("3.Cervena");
          Serial.println("4.Bila");
          Serial.println("5.Dolni zluta");
          Serial.println("6.Modra");
          Serial.println("7.Horni pruh/symbol");
          Serial.println("8.Dolni pruh");
          Serial.println("9.Ulozit do prijimace");
          Serial.println("0.Konec");
          Serial.println(NumSelection);
          do{
            MSelection=ReadChar();
          }while(MSelection<'0' || MSelection >'9');
          PrintSeparatorLine('-',65); 
          switch (MSelection){
            case '1':
              SetLight(UPPERYELLOW);
            break;
            case '2':
              SetLight(GREEN);
            break;
            case '3':
              SetLight(RED);
            break;
            case '4':
              SetLight(WHITE);
            break;
            case '5':
              SetLight(LOWERYELLOW);
            break;
            case '6':
              SetLight(BLUE);
            break;
            case '7':
              SetLight(UPPERVARIABLE);
            break;
            case '8':
              SetLight(LOWERSTRIP);
            break;
            case '9':
              SCOMTransmit(SAVE,Tb,REPEATPACKET); 
              Serial.println("OK = navest 1x blikne");
              Serial.println("CHYBA = navest trvale pomalu blika");
              PrintSeparatorLine('-',65);
            break;
            case '0':
              SCOMTransmit(ENDSETUP,Tb,REPEATPACKET); 
              Serial.println("Konec Setupu.");
            break;
            default:
            break;
          }
        }while(MSelection!='0');   
      }
//------------------------------------------------------------------------------------------------------------------     
  }while(true);
}
//*******************************************************************************************************************
void SetLight(byte LCode){ //nastavi jas svetla v prijimaci
  int BrightVal;
  Serial.println("Nastavovane svetlo blikne.");
  SCOMTransmit(LCode,Tb,REPEATPACKET); // posle kod nastavovaneho svetla
  do{
    Serial.println("Zadej jas 6-100[%] (>100 Exit)");
    do{
      BrightVal=ReadNum();
      if(BrightVal<6){
        Serial.println(InputError);
      }
    }while(BrightVal<6);  
    if(BrightVal<=100){      
      Serial.println(BrightVal);   
    }
    if(BrightVal<=100){
      SCOMTransmit(BrightVal,Tb,REPEATPACKET);  // posle hodnotu jasu  
    }
  }while(BrightVal<=100);  
  SCOMTransmit(SETUP,Tb,REPEATPACKET); // posle kod SETUP pro ukonceni nastaveni zvoleneho svetla
  
}

//*******************************************************************************************************************
void PrintSeparatorLine(char OutChar,byte CharNum){// posle na seriovou konzoli znak OutChar v poctu CharNum a odradkuje
  for(byte i=1;i<=CharNum;i++){
    Serial.print(OutChar);
  }
  Serial.println();
}

//*******************************************************************************************************************
int ReadNum(){ //precte cislo ze serioveho portu v rozsahu 0-999, max tri cislice bez znamenka, dalsi se budou ignorovat, pri neplatnem cteni vraci -1
  char ch=0,ChBuff[3]={0,0,0};
  byte NumDigit=0;
  int ReadValue=0;
  while(Serial.available()==0){  //cekame na znaky 
    ;
  }
  do{
    if (Serial.available() > 0) {
      ch=Serial.read();  
      if (ch>='0' && ch<='9'){
        ChBuff[NumDigit]=ch-'0';
        NumDigit++;      
      }
    }
  }while((ch>='0' && ch<='9')&&(NumDigit<3));//cteme, dokud to neni cislice nebo dve cislice
  if(NumDigit==0){//znak nebyl cislice,konec
    return -1;
  }
  if(NumDigit==1){
    ReadValue=ChBuff[0];
  }
  if(NumDigit==2){
    ReadValue=(ChBuff[0]*10)+ChBuff[1];
  }
  if(NumDigit==3){
    ReadValue=(ChBuff[0]*100)+(ChBuff[1]*10)+ChBuff[2];
  }  
  delay(20);
  while(Serial.available() > 0){//docteni zbytku
    Serial.read();
    delay(1);
  }  
  return ReadValue;
}

//*******************************************************************************************************************
char ReadChar(){ //cte ze seriove linky dokud nenarazi na prvni pismeno nebo cislici, pak docte do eol a to zahodi, vraci velkou hodnotu pismena nebo cislici
  char ch=-1;
  do{
    if (Serial.available() > 0) {
      ch=Serial.read();  
      if(ch>='a' && ch<='z'){ //uprava maleho pismena na velke
        ch=(ch-'a')+'A';
      }
    }
  }while(!((ch>='A' && ch<='Z')||(ch>='0' && ch<='9')));//cteme, dokud to neni pozadovany znak
  delay(20);
  while(Serial.available() > 0){//docteni zbytku
    Serial.read();
    delay(1);
  }
  return ch;
}

//*******************************************************************************************************************
void SCOMTransmit (byte Data,byte BitLen,byte RepeatTransmit){   //opakovane vysle zadany bit SCOM protokolem s definovanou delkou bitu v ms, vcetne meziblokove mezery
  byte Mask;
  Data=Data & 127;  //MSB na nulu
  for(byte n=0;n<RepeatTransmit;n++){
    Mask=1;
    digitalWrite(SCOMOUTPUT,LOW);  //startbit Low
    delay(BitLen);
    digitalWrite(SCOMOUTPUT,HIGH); //startbit High
    delay(BitLen);
    for (byte i=0; i<8;i++){
      if((Mask & Data)>0){
        digitalWrite(SCOMOUTPUT,HIGH); //vysilany bit je 1
      }else{
        digitalWrite(SCOMOUTPUT,LOW);  //vysilany bit je 0
      }
      Mask=Mask<<1;
      delay(BitLen);
    }
    digitalWrite(SCOMOUTPUT,HIGH);
    delay((10*BitLen)+ADDPACKETDISTANCE);
  }
}
